<?php
namespace Controller;

use Util\Utilities;
use Util\Messages;
use Zeedhi\Framework\Controller\Simple;
use Zeedhi\Framework\DataSource\DataSet;
use Zeedhi\Framework\DTO\Request;
use Zeedhi\Framework\DTO\Response;
use Zeedhi\Framework\DTO\Row;
use Zeedhi\DevOps\Util\Utilities as DevOpsUtilities;

class Appropriation extends Simple {

	/** @var \Service\Appropriation $appropriationService  */
	protected $appropriationService;

	public function __construct(\Service\Appropriation $appropriationService, \Service\Attachment $attachmentService) {
        $this->appropriationService = $appropriationService;
        $this->attachmentService = $attachmentService;
    }

    public function findAppropriation(Request\Filter $request, Response $response){
        $conditions = $request->getFilterCriteria()->getConditions();
        $appropriationId = empty($conditions[0]) ? 0 : $conditions[0]['value'];
        try {
            $appropriations = $this->appropriationService->getAppropriation($appropriationId);
            $response->addDataSet(new DataSet('appropriation', $appropriations));
        } catch (\Exception $e) {
            $response->addMessage(new Response\Message('Não foi possível achar sua apropriação, pois ' . $e->getMessage()));
        }
    }
    public function findAllAppropriations(Request\Filter $request, Response $response){
        $filters = $request->getFilterCriteria()->getConditions();
        try {
            $appropriations = empty($filters) ? array() : $this->appropriationService->findAllAppropriations($filters);
            DevOpsUtilities::setDataSetDefault($request, $response, $appropriations);
        } catch (\Exception $e) {
            $response->addMessage(new Response\Message('Não foi possível achar sua apropriação, pois ' . $e->getMessage()));
        }
    }
    public function findAllAppropriationsByDate(Request\Filter $request, Response $response){
        $filters = $request->getFilterCriteria()->getConditions();
        try {
            $appropriations = $this->appropriationService->findAllAppropriationsByDate($filters);
            DevOpsUtilities::setDataSetDefault($request, $response, $appropriations);
        } catch (\Exception $e) {
            $response->addMessage(new Response\Message('Não foi possível achar sua apropriação, pois ' . $e->getMessage()));
        }
    }

    public function removeAppropriations(Request\DataSet $request, Response $response){
        $appropriations = $request->getDataSet()->getRows();
        try {
            foreach ($appropriations as $appropriation) {
                $this->appropriationService->removeAppropriation($appropriation['ID']);
            }
            $response->addNotification(new Response\Notification(Messages::SUC_SUCCESSFUL_REMOVED));
        } catch (\Exception $e) {
            $response->addMessage(new Response\Message('Não foi possível remover a apropriação, pois ' . $e->getMessage()));
        }
    }

	public function saveAppropriation(Request\DataSet $request, Response $response) {
        try {
            $appropriationData = $request->getDataSet()->getRows();
            $appropriation = $this->persistAppropriation($appropriationData[0]);
            $id = $appropriation->getId();
            $appropriationData[0]['ID'] = $id;
            $response->addDataSet(new DataSet('appropriation', $appropriationData));
            $response->addNotification(new Response\Notification(Messages::SUC_SUCCESSFUL_SAVED));
            $emailsFailedToSend = Utilities::getCommentService()->emailsFailedToSend;
            if($emailsFailedToSend){
                    $emailsFailedToSend = implode(', ', $emailsFailedToSend);
                    $response->addMessage(new Response\Message('Especificação foi adicionada com sucesso mas não foi possivel enviar email para os seguintes endereços:  ' . $emailsFailedToSend));
                }
        } catch (\Exception $e) {
            $response->addMessage(new Response\Message(\Util\Messages::ERR_APPROPRIATION_NOT_SAVED . $e->getMessage(), Response\Message::TYPE_ERROR));
        }
	}

    public function persistAppropriation($appropriation) {
        try {
            $idAppropriation  = empty($appropriation['ID']) ? null : $appropriation['ID'];
            $specificationComment     = empty($appropriation['SPECIFICATION_COMMENT'])? null : $appropriation['SPECIFICATION_COMMENT'];
            $specificationCommentType    = empty($appropriation['SPECIFICATION_COMMENT_TYPE'])? null : $appropriation['SPECIFICATION_COMMENT_TYPE'];
            $specificationDocument    = empty($appropriation['SPECIFICATION_DOCUMENT'])? null : $appropriation['SPECIFICATION_DOCUMENT'];
            $specificationCdItemtrab    = empty($appropriation['SPECIFICATION_CDITEMTRAB'])? null : $appropriation['SPECIFICATION_CDITEMTRAB'];
            $idAllocation     = empty($appropriation['ALLOCATION'])? null : $appropriation['ALLOCATION'];
            $beginHour        = empty($appropriation['BEGIN_HOUR']) ? null : Utilities::parseStringToDate($appropriation['BEGIN_HOUR']);
            $endHour          = empty($appropriation['END_HOUR']) ? null : Utilities::parseStringToDate($appropriation['END_HOUR']);
            $total            = $appropriation['TOTAL'];
            $description      = $appropriation['DESCRIPTION'];
            $percentCompleted = empty($appropriation['PERCENT_COMPLETED']) ? 1 : (int)$appropriation['PERCENT_COMPLETED'];


                return $this->appropriationService->saveAppropriation($idAppropriation, $idAllocation, $beginHour, $endHour,
                $total, $description, $percentCompleted, $specificationComment,$specificationCommentType, $specificationDocument, $specificationCdItemtrab);

        } catch (\Exception $e) {
            throw $e;
        }
    }

    public function deleteAppropriation(Request\DataSet $request,Response $response){
        $data = $request->getDataSet()->getRows();
        $appropriationId = $data[0]['APPROPRIATION']['ID'];
        if (!empty($appropriationId)) {
            try {
                $this->appropriationService->deleteAppropriationById($appropriationId);
                $response->addNotification(new Response\Notification(Messages::SUC_SUCCESSFUL_REMOVED));
            } catch (\Exception $e) {
                $response->addMessage(new Response\Message($e->getMessage(), Response\Message::TYPE_WARNING));
            }
        }
    }

    public function getBeginHourAppropriation(Request\Filter $request,Response $response){
        $filter = $request->getFilterCriteria()->getConditions();
        $taskId = $filter[0]['value'];
        if (!empty($taskId)) {
            try {
                $dataBegin = $this->appropriationService->getBeginHourAppropriation($taskId);
                $response->addDataSet(new DataSet('dataBeginAppropriation', $dataBegin));
            } catch (\Exception $e) {
                $response->addMessage(new Response\Message($e->getMessage(), Response\Message::TYPE_WARNING));
            }
        }
    }

    /**
     * @param Request\DataSet $request
     * @param Response $response
     * @throws \Exception
     */
    public function createTaskBranchInDevOps(Request\DataSet $request, Response $response){
        $data = $request->getDataSet()->getRows()[0];
        $taskId = $data['TASK'];
        $currentTaskId = $data['CURRENT_TASK_ID'];
        $originProductVersionId = $data['ORIGIN_PRODUCT_VERSION_ID'];
        $forceCreateBranch = empty($data['forceCreateBranch']) ? null : $data['forceCreateBranch'];
        try {
            $this->appropriationService->insertDevOpsBranch($taskId, $currentTaskId, $forceCreateBranch, $originProductVersionId);
        } catch (\Exception $e) {
            $response->addMessage(new Response\Message($e->getMessage(), Response\Message::TYPE_ERROR));
            throw $e;
        }
    }
    /**
     * @param Request\Filter $request
     * @param Response $response
     * @throws \Exception
     */
    public function checkSameDirectoryFromDifferentRepositories(Request\Filter $request, Response $response)
    {
        $filter = $request->getFilterCriteria()->getConditions();
        $taskId = Utilities::getValueInFilterDataSourceByName($filter, 'TASK');
        $typeRelationship = Utilities::getValueInFilterDataSourceByName($filter, 'TYPE_RELATIONSHIP');
        $isWindows = Utilities::getValueInFilterDataSourceByName($filter, 'IS_WINDOWS');
        try {
            $projects = $this->appropriationService->checkSameDirectoryFromDifferentRepositories($taskId,$typeRelationship, $isWindows);
            $response->addDataSet(new DataSet('projects', $projects));
        } catch (\Exception $e) {
            $response->addMessage(new Response\Message($e->getMessage(), Response\Message::TYPE_ERROR));
            $response->setError(new Response\Error($e->getMessage(), $e->getCode()));
        }
    }

    /**
     * @param Request\Filter $request
     * @param Response $response
     * @throws \Exception
     */
    public function checkAllChangedFilesHaveLocalConnection(Request\Filter $request, Response $response)
    {
        $filter = $request->getFilterCriteria()->getConditions();
        $taskId = Utilities::getValueInFilterDataSourceByName($filter, 'TASK');
        $typeRelationship = Utilities::getValueInFilterDataSourceByName($filter, 'TYPE_RELATIONSHIP');

        try {
            $repositories = $this->appropriationService->checkAllChangedFilesHaveLocalConnection($taskId,$typeRelationship);
            $response->addDataSet(new DataSet('repositories', $repositories));
        } catch (\Exception $e) {
            $response->addMessage(new Response\Message($e->getMessage(), Response\Message::TYPE_ERROR));
            $response->setError(new Response\Error($e->getMessage(), $e->getCode()));
        }
    }

    /**
     * @param Request\Dataset $request
     * @param Response $response
     * @throws \Exception
     */
    public function createLogFile(Request\Dataset $request, Response $response)
    {
        try {
            $logRows = $request->getDataSet()->getRows();
            $log = $logRows[0];
            $logFileName = $log['FILE_NAME'];
            $logContents = $log['FILE_CONTENTS'];
            $this->appropriationService->createLogFile($logFileName, $logContents);

        } catch (\Exception $e) {
            $response->addMessage(new Response\Message($e->getMessage(), Response\Message::TYPE_ERROR));
            throw $e;
        }
    }

    /**
     * @param Request\Filter $request
     * @param Response $response
     */
    public function getLogFileContents(Request\Filter $request, Response $response)
    {
        $conditions = $request->getFilterCriteria()->getConditions();
        $logFileName = empty($conditions[0]) ? '' : $conditions[0]['value'];
        try {
            $logFileContents = $this->appropriationService->getLogFileContents($logFileName);
            $response->addDataSet(new DataSet('logFileContents', $logFileContents));
        } catch (\Exception $e) {
            $response->addMessage(new Response\Message($e->getMessage(), Response\Message::TYPE_ERROR));
        }
    }

    public function findDateLastAppropriation(Request\Filter $request, Response $response)
    {
        try {
            $dateLastAppropriation = $this->appropriationService->findDateLastAppropriation();
            $response->addDataSet(new DataSet('dateLastAppropriation', $dateLastAppropriation));
        } catch (\Exception $e) {
            $response->addMessage(new Response\Message($e->getMessage(), Response\Message::TYPE_ERROR));
        }
    }
}
