DROP TABLE TMP_COMPOSICAO_SPED
/

CREATE GLOBAL TEMPORARY TABLE TMP_COMPOSICAO_SPED
(
   CDPRODUTO       VARCHAR2(10)   NULL,
   QTDEPROD        FLOAT          NULL,
   NRNIVEL         NUMBER         NULL,
   CONSTRAINT PK_TMP_COMPOSICAO_SPED PRIMARY KEY(CDPRODUTO, NRNIVEL)) ON COMMIT PRESERVE ROWS
/

CREATE OR REPLACE PROCEDURE COMPOSICAO_PRODUTO_SPED (P_CDINSCESTA IN  EMPRESA.CDINSCESTA%TYPE,
                                                    P_CDPRODUTO IN  VARCHAR2,
                                                    P_NIVEL IN NUMBER)
IS
    V_CDFILIAL  VARCHAR2(4);
    V_COTPADRAO NUMBER;
    V_NIVEL     NUMBER;
    V_QTDEPROD  NUMBER;
BEGIN

   BEGIN
     SELECT MAX(R.CDFILIAL)
       INTO V_CDFILIAL
      FROM ROTFABLOCAL R, FILIAL F
     WHERE R.CDFILIAL = F.CDFILIAL
       AND F.CDINSCESTA = P_CDINSCESTA
       AND R.CDPRODUTO = P_CDPRODUTO;
   EXCEPTION
      WHEN NO_DATA_FOUND THEN
         V_CDFILIAL := 'X';
   END;

   V_NIVEL := P_NIVEL + 1;

   IF V_CDFILIAL <> 'X' THEN
      /* Composio Local */
      FOR C_COMPLOCAL IN (SELECT R.CDSUBPRODLOC, R.QTSUBPRODLOC
                            FROM ROTFABLOCAL R
                           WHERE R.CDFILIAL  = V_CDFILIAL
                             AND R.CDPRODUTO = P_CDPRODUTO
                           ORDER BY R.NRSEQPRODLOC)LOOP

           BEGIN
             SELECT COUNT(*)
               INTO V_QTDEPROD
               FROM TMP_COMPOSICAO_SPED
              WHERE CDPRODUTO = P_CDPRODUTO;
          EXCEPTION
             WHEN NO_DATA_FOUND THEN
                V_QTDEPROD := 0;
          END;

           IF V_QTDEPROD > 0 THEN
             DELETE FROM TMP_COMPOSICAO_SPED
                   WHERE CDPRODUTO = P_CDPRODUTO;
           END IF;

           INSERT INTO TMP_COMPOSICAO_SPED
             (CDPRODUTO, QTDEPROD, NRNIVEL)
           VALUES
             (C_COMPLOCAL.CDSUBPRODLOC, C_COMPLOCAL.QTSUBPRODLOC, V_NIVEL);

           COMPOSICAO_PRODUTO_SPED(P_CDINSCESTA, C_COMPLOCAL.CDSUBPRODLOC, V_NIVEL);
      END LOOP;
   ELSE
      BEGIN
        SELECT COUNT(*)
          INTO V_COTPADRAO
          FROM ROTFABRICA R
         WHERE R.CDPRODUTO = P_CDPRODUTO;
      EXCEPTION
           WHEN NO_DATA_FOUND THEN
              V_COTPADRAO := 0;
      END;
      
      IF V_COTPADRAO > 0 THEN
         /*Composio Padro*/
         FOR C_COMPPADRAO IN (SELECT R.CDSUBPRODUTO, R.QTSUBPRODROT
                                FROM ROTFABRICA R
                               WHERE R.CDPRODUTO  = P_CDPRODUTO
                               ORDER BY R.CDPRODUTO)LOOP

            BEGIN
               SELECT COUNT(*)
                 INTO V_QTDEPROD
                 FROM TMP_COMPOSICAO_SPED
                WHERE CDPRODUTO = P_CDPRODUTO;
            EXCEPTION
               WHEN NO_DATA_FOUND THEN
                  V_QTDEPROD := 0;
            END;
            
            IF V_QTDEPROD > 0 THEN
               DELETE FROM TMP_COMPOSICAO_SPED
                     WHERE CDPRODUTO = P_CDPRODUTO;
            END IF;

            INSERT INTO TMP_COMPOSICAO_SPED
               (CDPRODUTO, QTDEPROD, NRNIVEL)
            VALUES
               (C_COMPPADRAO.CDSUBPRODUTO, C_COMPPADRAO.QTSUBPRODROT, V_NIVEL);

            COMPOSICAO_PRODUTO_SPED(P_CDINSCESTA, C_COMPPADRAO.CDSUBPRODUTO, V_NIVEL);
        END LOOP;
      END IF;
   END IF;
END;
/

-- VERSAO --
BEGIN ATUVEROBJ('COMPOSICAO_PRODUTO_SPED','6.45.047'); END;
/
