
// FILE: src/config/config.js
(function () {

	var ZeedhiApp = angular.module("ZeedhiApp");
	
	window.templateUrls = window.templateUrls || [];
	var packageName = 'zh-task-gantt-editor';

	var hasPackageRegistered = window.templateUrls.some(function (templateUrl) {
		return templateUrl.package === packageName;
	});

	if (!hasPackageRegistered) {
		window.templateUrls.push({
			'package': packageName,
			'baseUrl': './bower_components/' + packageName + '/dist/templates/'
		});
	}
})();

Configuration(function (ContextRegister, eventEngine) {
	// register events 
});


// FILE: src/services/TaskGanttEditorEvents.js
function TaskGanttEditorEvents($rootScope) {
	var TaskGanttEditorWidget;
	
	this.registerWidget = function(widget) {
		TaskGanttEditorWidget = widget;
	};
	
	this.onChartFinishDrawing = function(args) {
		var chart = args.e.data.element;
		var chartAxis = chart.find('.x-axis');
		var chartAxisTop = chartAxis.offset().top;
		var chartAxisHeight = chartAxis[0].getBBox().height;
		
		var taskGanttEditor = chart.closest('.zh-task-gantt-editor-container');
		var taskGanttEditorTop = taskGanttEditor.offset().top;
		var treelist = taskGanttEditor.find('.treelist-container');
		var treelistColumnHeight = treelist.find('.zh-editable-treelist-column-header').outerHeight() || 0;
		var treelistToolbarHeight = treelist.find('.zh-editable-treelist-toolbar').outerHeight() || 0;
		
		treelist.css('margin-top', chartAxisTop - taskGanttEditorTop + chartAxisHeight - treelistColumnHeight - treelistToolbarHeight);
		
		$rootScope.$broadcast('gantt.finishDrawing');
	};
	
	this.onChartPanning = function(args) {
		$rootScope.$broadcast('gantt.panning', args.e.data);
	};
	
	this.onTreelistScroll = function(args) {
		$rootScope.$broadcast('treelist.scroll', args.e.data);
	};
	
	this.onAfterMoveRow = function(args) {
		$rootScope.$broadcast(args.owner.name == "GANTT_CHART" ? 'gantt.afterMoveRow' : 'treelist.afterMoveRow', args.row);

		if (TaskGanttEditorWidget.afterMoveRow) {
			TaskGanttEditorWidget.afterMoveRow(args);	
		}
	};
	
	this.toggleCollapsedItemsView = function() {
		TaskGanttEditorWidget.toggleCollapsedItemsView();	
	};
}

Configuration(function(ContextRegister) {
    ContextRegister.register('TaskGanttEditorEvents', TaskGanttEditorEvents);
});



// FILE: src/services/TaskGanttEditorWidgetsBuilder.js
/**
 * Service to create widgets
 *
 * @class
 * @since 0.0.1
 *
 * @param {RestEngine}      restEngine      Service to make http requests
 * @param {TemplateManager} templateManager Service to handle templates
 * @param {MetaDataFactory} metaDataFactory Service to factory models from metadata
 */
function TaskGanttEditorWidgetsBuilder(restEngine, templateManager, metaDataFactory) {

    function getWidgetJSON(widgetName) {
        var widgetPath = 'zh-task-gantt-editor#json/widgets/'+widgetName+'.json';
        return restEngine.requestMetaData(widgetPath).then(function(response) {
            return response.data;
        });
    }

    function getJSON(jsonName) {
        var jsonPath = 'zh-task-gantt-editor#json/'+jsonName+'.json';
        return restEngine.requestMetaData(jsonPath).then(function(response) {
            return response.data;
        });
    }

    /**
     * Retrieve a factored widget
     *
     * @since 0.0.1
     *
     * @param  {String} widgetName Widget name
     *
     * @return {Widget}            Factored widget
     */
    this.factoryWidget = function(widgetName) {
        return getWidgetJSON(widgetName).then(function(widget) {
            return metaDataFactory.widgetFactory(widget, templateManager.container);
        });
    };

    /**
     * Retrieve factored gantt chart widget
     *
     * @since 0.0.1
     *
     * @return {Widget} Gantt Chart widget
     */
    this.factoryGanttChartWidget = function() {
        return this.factoryWidget('gantt');
    };

    /**
     * Retrieve factored editable tree list widget
     *
     * @since 0.0.1
     *
     * @return {Widget} Editable tree list widget
     */
    this.factoryEditableTreeListWidget = function() {
        return this.factoryWidget('treelist');
    };
    
    /**
     * Retrieve floating control JSON
     *
     * @since 0.0.1
     *
     * @return {Object} Floating control JSON
     */
    this.getFloatingControlJSON = function() {
        return getJSON('floating-control').then(function(json) {
            return json;
        });
    };
}

Configuration(function(ContextRegister) {
    ContextRegister.register('TaskGanttEditorWidgetsBuilder', TaskGanttEditorWidgetsBuilder);
});



// FILE: src/controllers/TaskGanttEditorController.js
function TaskGanttEditorController($scope, ScreenService, ApplicationContext, eventEngine) {

	// BEGIN: Initialize
 	var self = this;
 	var widget = $scope.widget;
 	var TaskGanttEditorWidgetsBuilder = ApplicationContext.TaskGanttEditorWidgetsBuilder;
 	var TaskGanttEditorEvents = ApplicationContext.TaskGanttEditorEvents;
 	TaskGanttEditorEvents.registerWidget(widget);

 	buildWidgets();
	// END: Initialize

	function buildWidgets() {
		TaskGanttEditorWidgetsBuilder.factoryGanttChartWidget().then(function(ganttWidget) {
			$scope.ganttWidget = ganttWidget;

			$scope.ganttWidget.chartSeries[0].serieGanttFields = widget.taskFields;
			$scope.ganttWidget.chartSeries[0].serieGanttRealBarType = widget.ganttRealBarType;
			$scope.ganttWidget.chartSeries[0].serieGanttTimeScale = widget.ganttTimeScale;
			$scope.ganttWidget.chartSeries[0].serieGanttShowPredecessors = widget.ganttShowPredecessors;
			$scope.ganttWidget.chartSeries[0].serieGanttDateFormat = widget.ganttDateFormat;
			$scope.ganttWidget.chartSeries[0].serieGanttItemBarHeight = widget.ganttItemBarHeight;
			$scope.ganttWidget.chartSeries[0].serieGanttShowTaskCompletionLabel = widget.showTaskCompletionLabel;
			$scope.ganttWidget.chartSeries[0].serieGanttShowCompletionLabelOnParentTasks = widget.showCompletionLabelOnParentTasks;
			$scope.ganttWidget.chartSeries[0].serieGanttShowTaskName = widget.showTaskName;
			$scope.ganttWidget.chartSeries[0].serieGanttShowCompletedTasks = widget.showCompletedTasks;
			$scope.ganttWidget.chartSeries[0].serieGanttOverrideTaskColorEstimated = widget.overrideTaskColorEstimated;
			$scope.ganttWidget.chartSeries[0].serieGanttOverrideTaskColorReal = widget.overrideTaskColorReal;
		});

		TaskGanttEditorWidgetsBuilder.factoryEditableTreeListWidget().then(function(treelistWidget) {
			$scope.treelistWidget = treelistWidget;

			$scope.treelistWidget.primaryProperty    = widget.taskFields.taskId;
			$scope.treelistWidget.parentProperty     = widget.taskFields.taskParent;
			$scope.treelistWidget.expandingProperty  = widget.taskFields.taskName;
			$scope.treelistWidget.breadcrumbProperty = widget.taskFields.taskName;
			$scope.treelistWidget.itemColorProperty  = widget.taskFields.taskColor;
			$scope.treelistWidget.isParentProperty   = widget.taskFields.taskIsParent;
			$scope.treelistWidget.overrideTaskColor  = widget.overrideTaskColor;
			$scope.treelistWidget.taskColor          = widget.taskColorOnTreeList;

			$scope.treelistWidget.showColumns = widget.showTreelistColumns;

			$scope.treelistWidget.fields = widget.fields;
			$scope.treelistWidget.dataSource = widget.dataSource;

			$scope.treelistWidget.filterDataFunction = function(row) {
				return widget.showCompletedTasks !== false || row[widget.taskFields.taskCompleted] < 100;
			};

			$scope.$watchCollection('treelistWidget.plainTreeData', function(newValue, oldValue) {
				if (newValue && $scope.ganttWidget) {
					var chart = $scope.ganttWidget.chart;
					if (chart) {
						chart.clear();
					}

					$scope.ganttWidget.dataSource.data = newValue.filter($scope.treelistWidget.filterDataFunction);
				}
			});
		});
	}

	self.isParent = function(row) {
		return row[widget.taskFields.taskIsParent] === true || row[widget.taskFields.taskIsParent] == 'Y' || widget.dataSource.data.some(function(dataRow) {
			return dataRow[widget.taskFields.taskParent] == row[widget.taskFields.taskId];
		});
	};

	$scope.$on('$destroy', function() {
		delete $scope.treelistWidget.filterDataFunction;
		delete $scope.treelistWidget;
		delete $scope.ganttWidget;
		delete widget.toggleCollapsedItemsView;
	});

	$scope.$watchCollection('widget.dataSource.data', function(newValue, oldValue) {
		if (oldValue && newValue) {
			oldValue.filter(function(item) {
				return item._opened;
			}).forEach(function(oldItem) {
				var newRow = newValue.find(function(newItem) {
					return newItem[widget.taskFields.taskId] == oldItem[widget.taskFields.taskId];
				});
				if (newRow) {
					newRow._opened = true;
				}
			});
		}
	});

	widget.toggleCollapsedItemsView = function() {
		$scope.treelistWidget.toggleCollapsedItemsView();
	};
}



// FILE: src/controllers/TaskGanttEditorFloatingController.js
function TaskGanttEditorFloatingController($scope) {

	//------ Control variables
	var editingWidget = $scope.$parent.widget;

	/**
	 * Checks if the card is already opened and reconfigure the state of it if it not.
	 *
	 * @param  {jquery.Event} e   Event that triggered the click.
	 */
	function checkFloatingCardOverflow(e) {
		var actionElement = $(e.target).closest(".float-action");
		var floatingControl = actionElement.parents(".zh-floating-control");
		var widgetContainer = actionElement.closest(".container");
		var actionTop, actionBottom;
		var widgetTop, widgetBottom;

		if (!actionElement.hasClass("opened")) {
			actionElement.removeClass("fix-position-top");
			actionElement.removeClass("fix-position-bottom");

			actionTop = actionElement.offset().top;
			widgetTop = widgetContainer.offset().top;
			widgetBottom = widgetTop + widgetContainer.height();
			if (floatingControl.hasClass("pos-top")) {
				actionTop = actionTop - 200 + actionElement.height();
				if (actionTop < widgetTop) {
					actionElement.addClass("fix-position-top");
				}
			}
			else {
				actionBottom = actionTop + 200;
				if (actionBottom > widgetBottom) {
					actionElement.addClass("fix-position-bottom");
				}
			}
		}
	}

	/**
	 * Toggle the report action when it's template is configured as card.
	 *
	 * @param  {jquery.Event} e   Event that triggered the toggle.
	 */
	this.toggleFloatCard = function(e) {
		checkFloatingCardOverflow(e);
		$scope.toggleAction(e);
	};

	this.actionOptionClick = function(option, widget) {
		var chart = widget.getGanttChart();
		switch (option.id) {
			case 'EXPORT_SVG':
				chart.saveSVG();
				break;
			case 'EXPORT_PNG':
				chart.savePNG();
				break;
			case 'SCALE_DAY':
				chart.ganttTimeScale('day').redraw();
				break;
			case 'SCALE_WEEK':
				chart.ganttTimeScale('week').redraw();
				break;
			case 'SCALE_MONTH':
				chart.ganttTimeScale('month').redraw();
				break;
			case 'SCALE_QUARTER':
				chart.ganttTimeScale('quarter').redraw();
				break;
			case 'SCALE_YEAR':
				chart.ganttTimeScale('year').redraw();
				break;
		}
	};

	editingWidget.toggleFloatReportCard = $scope.toggleFloatReportCard;
}

// FILE: src/directives/zhTaskGanttEditor.js
var ZeedhiDirectives = angular.module("ZeedhiDirectives");

ZeedhiDirectives.directive('zhTaskGanttEditor', function($rootScope, $timeout, ApplicationContext) {
	return {
		link: function($scope, element, attrs, controller) {
			var widget = $scope.widget;

			var chartContainer = element.find('.gantt-chart-container');
			var treelistContainer = element.find('.treelist-container');
			var gutter = element.find('.task-gantt-gutter');
			var lastHeight;

			var elementWidth;
			var chartWidth, treelistWidth;

			function resizeElements() {
				var gutterLeft = parseFloat(gutter.css('left'));
				elementWidth = element.width();

				if (!widget.sizeable || gutterLeft === 0) {
					if (widget.chartGanttWidth) {
						if (widget.chartGanttWidth.toString().indexOf('%') !== -1) {
							chartWidth = elementWidth * parseInt(widget.chartGanttWidth) / 100;
						} else {
							chartWidth = parseInt(widget.chartGanttWidth);
						}
					} else {
						chartWidth = elementWidth / 2;
					}
					treelistWidth = elementWidth - chartWidth;

					if (widget.sizeable) {
						treelistWidth -= gutter.width();
						gutter.css('left', treelistWidth);
					}
				} else {
					treelistWidth = gutterLeft;
					chartWidth = elementWidth - gutterLeft - gutter.width();
				}

				chartContainer.width(chartWidth);
				treelistContainer.width(treelistWidth);
			}
			resizeElements();
			$(window).on('resize.taskgantteditor', resizeElements);

			gutter.on('touchstart mousedown', function() {
				gutter[0]._dragging = true;
			});

			$(window).on('touchend.taskgantteditor mouseup.taskgantteditor', function() {
				gutter[0]._dragging = false;
			});

			$(window).on('touchmove.taskgantteditor mousemove.taskgantteditor', function(e) {
				if (gutter[0]._dragging) {
					var deltaX = e.originalEvent.movementX;

					if (parseInt(gutter.css('left')) + deltaX > element.width() * 0.1 && parseInt(gutter.css('left')) + deltaX < element.width() * 0.9) {
						gutter.css('left', parseInt(gutter.css('left')) + deltaX);
						treelistContainer.width(treelistContainer.width() + deltaX);

						var zhChartContainer = chartContainer.find('.zh-chart-container');
						var chart = $scope.ganttWidget.chart;

						chartContainer.width(chartContainer.width() - deltaX);
						chart.redraw();
					}
				}
			});

			function setGanttItemHeight() {
				var treelistItem = treelistContainer.find('li.zh-treelist-item').eq(0);

				$timeout(function() {
					var treelistItemHeight = treelistItem.outerHeight();

					if (treelistItemHeight != lastHeight) {
						if ($scope.ganttWidget) {
							lastHeight = treelistItemHeight;
							var chart = $scope.ganttWidget.chart;

							chart.updateSerieOptions(0, { itemHeight: treelistItemHeight+5 }).updateSeries();
						}
					}
				});
			}

			var portRepeatDeregister = $rootScope.$on('treelist.postRepeat', setGanttItemHeight);
			var toggleCollapsedItemsViewDeregister = $rootScope.$on('treelist.toggleCollapsedItemsView', setGanttItemHeight);

			function chartPanning(event, panningData) {
				var editableTreelistContainer = treelistContainer.find('.zh-editable-treelist-container');
				if (editableTreelistContainer.scrollTop() != panningData.y) {
					editableTreelistContainer.scrollTop(panningData.y);
				}
			}

			var panningDeregister = $rootScope.$on('gantt.panning', chartPanning);

			function setChartCurrentRow(row) {
				var chart;
				if ($scope.ganttWidget) {
					chart = $scope.ganttWidget.chart;
					chart.currentRow = row;
					chart.updateSeries();
				}

				return chart;
			}

			function chartAfterMoveRow(event, row) {
				if ($scope.treelistWidget) {
					$scope.treelistWidget.currentRow = row;
				}

				setChartCurrentRow(row);
			}

			function treelistAfterMoveRow(event, row) {
				var chart = setChartCurrentRow(row);
				chart.ganttScrollToView(row);
			}

			var chartAfterMoveRowDeregister = $rootScope.$on('gantt.afterMoveRow', chartAfterMoveRow);
			var treelistAfterMoveRowDeregister  = $rootScope.$on('treelist.afterMoveRow', treelistAfterMoveRow);

			function treelistScroll(event, scrollY) {
				if ($scope.ganttWidget) {
					var chart = $scope.ganttWidget.chart;

					var chartPanningData = chart.panning();
					if (chartPanningData.y != scrollY) {
						chart.panning({x: -chartPanningData.x, y: scrollY});
					}
				}
			}

			var scrollDeregister = $rootScope.$on('treelist.scroll', treelistScroll);

			$scope.$on('$destroy', function() {
				portRepeatDeregister();
				toggleCollapsedItemsViewDeregister();
				finishDrawingDeregister();
				panningDeregister();
				chartAfterMoveRowDeregister();
				treelistAfterMoveRowDeregister();
				scrollDeregister();

				delete widget.getGanttChart;

				$(window).off('touchend.taskgantteditor mouseup.taskgantteditor');
				$(window).off('touchmove.taskgantteditor mousemove.taskgantteditor');
				$(window).off('resize.taskgantteditor');
			});

			widget.getGanttChart = function() {
				var chart;
				if ($scope.ganttWidget) {
					chart = $scope.ganttWidget.chart;
				}

				return chart;
			};

			var TaskGanttEditorWidgetsBuilder = ApplicationContext.TaskGanttEditorWidgetsBuilder;

			(function initFloatingControl() {
				TaskGanttEditorWidgetsBuilder.getFloatingControlJSON().then(function(fcontrol) {
					widget.floatingControl = widget.floatingControl || {};
					widget.floatingControl.actions = fcontrol.actions;
				});
			})();
		}
	};
});
